﻿using System;
using XGame.Framework.ObjectCollection;

namespace XGame.Framework.Map
{
    /// <summary>
    /// Entity实例
    /// 没有Controller
    /// </summary>
    /// <typeparam name="TViewModel"></typeparam>
    public abstract class EntityView<TViewModel> : IEntityView, IMapContextSetter, IEntityProxy, IDisposable where TViewModel : class, IEntityViewModel
    {
        private MapContext _context;
        protected MapContext Context => _context;

        private EntityBehaviour _behaviour;
        private IEntityViewModel _viewModel;
        protected TViewModel VM => _viewModel as TViewModel;

        private void Init()
        {
            _viewModel = Activator.CreateInstance<TViewModel>();
            (_viewModel as EntityViewModel)?.Init(_behaviour);
            //_controller = Activator.CreateInstance<TCtrl>();
            //if (_controller is IMapContextSetter contextSetter)
            //{
            //    contextSetter.Context = context;
            //}
            //if (_controller is IEntityViewModelSetter vmSetter)
            //{
            //    vmSetter.ViewModel = _viewModel;
            //}
            OnInited();
        }

        #region 接口实现

        MapContext IMapContextSetter.Context { set => _context = value; }
        EntityBehaviour IEntityProxy.Behaviour 
        { 
            get => _behaviour;
            set 
            {
                _behaviour = value; 
                Init();
            }
        }
        public bool Active { get; private set; }
        

        public void Enable(object intent)
        {
            if (Active) return;
            Active = true;
            _behaviour.gameObject.SetActive(true);
            OnEnable(intent);
        }


        public void Disable()
        {
            if (!Active) return;
            Active = false;
            _behaviour.gameObject.SetActive(false);
            OnDisable();
        }


        void IDisposable.Dispose()
        {
            OnDispose();
            (_viewModel as IDisposable)?.Dispose();
            _viewModel = null;
            _behaviour = null;
            _context = null;
        }
        #endregion

        protected abstract IEntityViewModel CtreatViewModel(IObjectCollector collector);

        protected abstract void OnEnable(object intent);

        protected abstract void OnDisable();
        protected virtual void OnInited()
        {

        }
        protected virtual void OnDispose()
        {
        }
    }
}