using System;
using UnityEngine;
using XGame.Framework.Asset;

namespace XGame.Framework.Map
{
    public class MapAssetModule : IMapAssetModule, IDisposable
    {
        private IAssetModule _assetModule;
        private MapContext _context;
        public MapAssetModule(MapContext mapContext)
        {
            _assetModule = new AssetModule();
            _context = mapContext;
        }
        IAssetLoadAsync<TResult> IMapAssetModule.LoadAsync<TResult>(string addressableName)
        {
            return _assetModule.LoadAsync<TResult>(addressableName);
        }
        void IMapAssetModule.LoadEntity<TEntityView>(string addressableName, Transform parent, Action<TEntityView> callback)
        {
            var loadAsync = _assetModule.LoadAsync<EntityBehaviour>(addressableName);
            loadAsync.On(_ =>
            {
                var behaviour = loadAsync.GetResult();
                if (behaviour == null)
                {
                    callback?.Invoke(null);
                    return;
                }
                var entityView = CreateEntity<TEntityView>(behaviour);
                if (parent != null)
                {
                    behaviour.transform.SetParent(parent, false);
                }
                callback?.Invoke(entityView);
            });
        }
        public TEntityView CreateEntity<TEntityView>(EntityBehaviour behaviour) where TEntityView : class, IEntityView, new()
        {
            var entityView = Activator.CreateInstance<TEntityView>();
            if (entityView is IMapContextSetter contextSetter)
            {
                contextSetter.Context = _context;
            }
            if (entityView is IEntityProxy proxy)
            {
                proxy.Behaviour = behaviour;
            }
            return entityView;
        }
        void IMapAssetModule.Recycle(UnityEngine.Object obj)
        {
            _assetModule.Recycle(obj);
        }
        void IMapAssetModule.Recycle(IEntityView entityView)
        {
            if (entityView is IEntityProxy proxy)
            {
                var behaviour = proxy.Behaviour;
                _assetModule.Recycle(behaviour);
            }
            (entityView as IDisposable)?.Dispose();
        }
        void IMapAssetModule.StopAllAsyncs()
        {
            _assetModule.StopAllAsyncs();
        }

        void IDisposable.Dispose()
        {
            (_assetModule as IDisposable)?.Dispose();
            _assetModule = null;
        }
    }
}
